/*
 * lodju.h - master header file for the Lodju project
 *
 * Every source module is supposed to include this header.
 */


#ifndef LODJU_H
#define LODJU_H


/*
 * Includes for system headers needed to use stuff in this header.
 */
#include <stdio.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

/*
 * Includes for i18n - optional.
 * Also defines the _ macro for message catalog lookups or, if
 * i18n is not to be used, defines a fallback
 */

#if HAVE_LIBINTL_H
    #include <libintl.h>
    #define _(x) gettext(x)
#else
    #define _(x) x
#endif
#define N_(x) x

/*
 * Includes for other project headers.
 */
#include "imagedb.h"
#include "pixbuf.h"


/*
 * Unit testing macros.
 */
#define TEST(name, statement) \
    do { \
    	printf("  Test %s\n", name); \
	setup(); \
	statement \
	teardown(); \
    } while (0)
#define ASSERT(condition) \
    do { \
    	if (!(condition)) { \
	    printf("%s:%d:TEST FAILED:%s\n", __FILE__, __LINE__, #condition); \
	    return -1; \
	} \
    } while (0)


/***********************************************************************
 * Constants for drag-and-drop mime types.
 */


enum {
    MIME_LODJU_DND_IMAGE,
    MIME_LODJU_DND_FOLDER
};


/***********************************************************************
 * Configuration interface.
 */


void cfg_load(void);
glong cfg_get_cache_size(void);


/***********************************************************************
 * Meta data about image.
 */


void meta_set_widgets(GladeXML *xml);

void meta_draw_image(void);
void meta_clear(void);
void meta_fill(Image *image);
void meta_set_progress_bar(gfloat percentage);


/***********************************************************************
 * Folder tree.
 */


void folder_tree_set(GtkCTree *ctree);
GtkCTree *folder_tree_get(void);
ImageFolder *folder_tree_get_selected(void);
GtkCTreeNode *folder_tree_get_node(void);
void folder_tree_set_node(GtkCTreeNode *node);

void folder_tree_clear(void);
void put_folder_tree_into_widget(ImageFolder *folder, 
    	    	    	    	    	GtkCTree *folder_tree,
    	    	    	    	        GtkCTreeNode *parent_node);



/***********************************************************************
 * Image list.
 */


/* Dimensions of the thumbnails in the image list. */
enum {
    THUMBNAIL_WIDTH = 32,
    THUMBNAIL_HEIGHT = 32
};


void image_list_set(GtkCList *clist);
GtkCList *image_list_get(void);
Image *image_list_get_selected(void);
void image_list_clear(void);
void image_list_append_image(Image *image);
void image_list_set_attribute(const gchar *name, const gchar *value);
void image_list_draw_thumbnails(void);
void image_list_remove(void);

void on_next_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_previous_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_image_list_select_row(GtkCList *clist, gint row, gint column,
    	    	    	      GdkEventButton *event, gpointer user_data);
void on_image_list_unselect_row(GtkCList *clist, gint row, gint column,
				GdkEventButton *event, gpointer user_data);


/***********************************************************************
 * Toplevel window.
 */


void toplevel_set_widgets(GladeXML *xml);
void open_image_database(const gchar *filename);
void save_image_database(void);
void close_image_database(void);
void on_import_ok_button_clicked(GtkButton *button, gpointer user_data);
void on_import_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_remove_image_activate(GtkMenuItem *menuitem, gpointer user_data);
void on_save_database_activate(GtkMenuItem *menuitem, gpointer user_data);
gboolean on_lodju_window_delete_event(GtkWidget *widget, GdkEvent *event, 
    	    	    	    	      gpointer p);
void on_exit_activate(GtkMenuItem *menuitem, gpointer user_data);



/***********************************************************************
 * Utility functions in utils.c.
 */


/*
 * Comparison function for g_list_sort: compare basenames of two filenames.
 */
gint compare_basenames(gconstpointer a, gconstpointer b);


/*
 * Return list (sorted by basename) of filenames in `directory' that are
 * image files. The name of the directory is prepended to the names.
 */
GList *find_images(const gchar *directory);


/*
 * Initialize the gettext library.
 */
void gettext_init(void);


/*
 * Load the Lodju Glade file for a given root widget.
 */
GladeXML *load_user_interface(const gchar *root_widget);


/*
 * Unit tests.
 */
gint utils_unit_test(void);


/*
 * XML parsing utilities.
 */
#include <gnome-xml/tree.h>
gchar *node_get_content(xmlNodePtr node);
gchar *node_get_attribute(xmlNodePtr node, const gchar *name);



#endif
