/*
 * toplevel.c - signal handlers, etc., for the toplevel window
 */


#include "lodju.h"


static GtkFileSelection *import_fileselection;
static gchar *import_selection;
static GtkFileSelection *open_fileselection;
static gchar *open_selection;
static GtkFileSelection *save_as_fileselection;
static ImageDB *imagedb;
static GtkWindow *toplevel_window;


static void set_title(const gchar *filename)
{
    gchar *title;
    
    if (filename == NULL)
    	filename = _("Unknown");
    title = g_strdup_printf("Lodju: %s", filename);
    gtk_window_set_title(toplevel_window, title);
    g_free(title);
}


void toplevel_set_widgets(GladeXML *xml)
{
    toplevel_window = 
    	GTK_WINDOW(glade_xml_get_widget(xml, "lodju_window"));
}


void open_image_database(const gchar *filename)
{
    set_title(filename);
    close_image_database();
    imagedb = image_db_create(filename);
    folder_tree_clear();
    put_folder_tree_into_widget(image_db_get_root(imagedb), folder_tree_get(), 
    	    	    	    	NULL);
}


void save_image_database(void)
{
    if (image_db_is_modified(imagedb))
	image_db_save(imagedb);
}


void close_image_database(void)
{
    image_db_destroy(imagedb);
    imagedb = NULL;
}


static void connect_widget_destroy(GtkButton *button, gpointer dialog)
{
    gtk_signal_connect_object(GTK_OBJECT(button),
			      "clicked", 
			      GTK_SIGNAL_FUNC(gtk_widget_destroy),
			      dialog);
}


static void start_triple_button_dialog(const gchar *name, 
				       const gchar *yes_name, 
				       GtkSignalFunc do_yes,
				       const gchar *no_name, 
				       GtkSignalFunc do_no,
				       const gchar *cancel_name)
{
    GladeXML *xml;
    GtkDialog *dialog;
    GtkButton *yes;
    GtkButton *no;
    GtkButton *cancel;
    
    xml = load_user_interface(name);
    if (xml == NULL)
	return;

    dialog = GTK_DIALOG(glade_xml_get_widget(xml, name));
    yes = GTK_BUTTON(glade_xml_get_widget(xml, yes_name));
    no = GTK_BUTTON(glade_xml_get_widget(xml, no_name));
    cancel = GTK_BUTTON(glade_xml_get_widget(xml, cancel_name));

    if (do_yes != NULL)
	gtk_signal_connect(GTK_OBJECT(yes), "clicked", do_yes, NULL);
    if (do_no != NULL)
	gtk_signal_connect(GTK_OBJECT(no), "clicked", do_no, NULL);

    connect_widget_destroy(yes, dialog);
    connect_widget_destroy(no, dialog);
    connect_widget_destroy(cancel, dialog);

    gtk_widget_show(GTK_WIDGET(dialog));
}



static void ask_save_database(GtkSignalFunc do_yes, GtkSignalFunc do_no)
{
    start_triple_button_dialog("save_database_dialog",
    	    	    	       "save_database_yes", do_yes,
    	    	    	       "save_database_no", do_no,
			       "save_database_cancel");
}


static GtkFileSelection *show_fileselection(const gchar *name,
    	    	    	    	    	    const gchar *selection)
{
    GladeXML *xml;
    GtkFileSelection *fs;
    gchar *dirname;
    
    xml = load_user_interface(name);
    if (xml == NULL)
	return NULL;
    fs = GTK_FILE_SELECTION(glade_xml_get_widget(xml, name));
    if (selection != NULL) {
	dirname = g_strdup_printf("%s/", selection);
	gtk_file_selection_set_filename(fs, dirname);
	g_free(dirname);
    }
    connect_widget_destroy(GTK_BUTTON(fs->ok_button), fs);
    connect_widget_destroy(GTK_BUTTON(fs->cancel_button), fs);
    gtk_widget_show(GTK_WIDGET(fs));
    return fs;
}


void on_open_ok_button_clicked(GtkButton *button, gpointer user_data)
{
    const gchar *filename;

    filename = gtk_file_selection_get_filename(open_fileselection);
    g_free(open_selection);
    open_selection = g_dirname(filename);
    open_image_database(filename);
}


static void open_database_yes(GtkButton *button, gpointer user_data)
{
    save_image_database();
    open_fileselection = show_fileselection("open_fileselection",
					    open_selection);
}


static void open_database_no(GtkButton *button, gpointer user_data)
{
    open_fileselection = show_fileselection("open_fileselection",
					    open_selection);
}


void on_open_database_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    if (image_db_is_modified(imagedb))
    	ask_save_database(open_database_yes, open_database_no);
    else
	open_fileselection = show_fileselection("open_fileselection",
						open_selection);
}


static void free_image_name(gpointer name, gpointer dummy)
{
    g_free(name);
}


void on_import_ok_button_clicked(GtkButton *button, gpointer user_data)
{
    GList *image_names;
    GList *item;
    const gchar *filename;
    
    filename = gtk_file_selection_get_filename(import_fileselection);
    g_free(import_selection);
    import_selection = g_dirname(filename);
    image_names = find_images(filename);
    
    /* Display info */
    if (folder_tree_get_selected() != NULL) {
	for (item = image_names; item != NULL; item = item->next) {
	    image_list_append_image(image_folder_import(
		    	    folder_tree_get_selected(), item->data));
	}
    }
    
    g_list_foreach(image_names, free_image_name, NULL);
    g_list_free(image_names);
}


void on_import_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    import_fileselection = show_fileselection("import_fileselection",
    	    	    	    	    	      import_selection);
}


void on_remove_image_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    image_list_remove();
}


void on_save_as_ok_button_clicked(GtkButton *button, gpointer user_data)
{
    const gchar *filename;
    
    filename = gtk_file_selection_get_filename(save_as_fileselection);
    image_db_set_name(imagedb, filename);
    set_title(filename);
    save_image_database();
}


void on_save_as_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    save_as_fileselection = show_fileselection("save_as_fileselection",
    	    	    	    	    	       image_db_get_name(imagedb));
}


void on_save_database_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    if (image_db_get_name(imagedb) == NULL)
    	on_save_as_activate(NULL, NULL);
    else
	image_db_save(imagedb);
}


static void new_database_yes(GtkButton *button, gpointer user_data)
{
    save_image_database();
    open_image_database(NULL);
}


void new_database_no(GtkButton *button, gpointer user_data)
{
    open_image_database(NULL);
}


void on_new_database_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    if (image_db_is_modified(imagedb))
    	ask_save_database(new_database_yes, new_database_no);
    else
	open_image_database(NULL);
}


void on_exit_save_yes_clicked(GtkButton *button, gpointer user_data)
{
    save_image_database();
    gtk_main_quit();
}


void on_exit_save_no_clicked(GtkButton *button, gpointer user_data)
{
    gtk_main_quit();
}


static void ask_save_or_quit(void)
{
    if (image_db_is_modified(imagedb)) {
    	start_triple_button_dialog("exit_save",
	    	    	    	   "exit_save_yes", NULL,
				   "exit_save_no", NULL,
				   "exit_save_cancel");
    } else
    	gtk_main_quit();
}


gboolean on_lodju_window_delete_event(GtkWidget *widget, GdkEvent *event, 
    	    	    	    	      gpointer p)
{
    ask_save_or_quit();
    return TRUE;
}


void on_exit_activate(GtkMenuItem *menuitem, gpointer user_data)
{
    ask_save_or_quit();
}
